@@ -1581,16 +1581,16 @@ class JMeter(RequiredTool):
         # component name and download link in https://repo1.maven.org/maven2/
         affected_components = {
             # Needs to be <1.4.18 for old Jmeters https://stackoverflow.com/questions/30812293/com-thoughtworks-xstream-security-forbiddenclassexception
-            "xstream": "com/thoughtworks/xstream/xstream/1.4.19/xstream-1.4.19.jar",
-            "jackson-annotations": "com/fasterxml/jackson/core/jackson-annotations/2.13.4/jackson-annotations-2.13.4.jar",
-            "jackson-core": "com/fasterxml/jackson/core/jackson-core/2.13.4/jackson-core-2.13.4.jar",
-            "jackson-databind": "com/fasterxml/jackson/core/jackson-databind/2.13.4.2/jackson-databind-2.13.4.2.jar",
+            "xstream": "com/thoughtworks/xstream/xstream/1.4.20/xstream-1.4.20.jar",
+            "jackson-annotations": "com/fasterxml/jackson/core/jackson-annotations/2.14.1/jackson-annotations-2.14.1.jar",
+            "jackson-core": "com/fasterxml/jackson/core/jackson-core/2.14.1/jackson-core-2.14.1.jar",
+            "jackson-databind": "com/fasterxml/jackson/core/jackson-databind/2.14.1/jackson-databind-2.14.1.jar",
             "json-smart": "net/minidev/json-smart/2.4.8/json-smart-2.4.8.jar",
             "jsoup": "org/jsoup/jsoup/1.15.3/jsoup-1.15.3.jar",
             "snakeyaml": "org/yaml/snakeyaml/1.33/snakeyaml-1.33.jar",
             "okhttp": "com/squareup/okhttp3/okhttp/4.10.0/okhttp-4.10.0.jar",
             "commons-text": "org/apache/commons/commons-text/1.10.0/commons-text-1.10.0.jar",
-            "xmlgraphics-commons": "org/apache/xmlgraphics/xmlgraphics-commons/2.7/xmlgraphics-commons-2.7.jar"}
+            "xmlgraphics-commons": "org/apache/xmlgraphics/xmlgraphics-commons/2.8/xmlgraphics-commons-2.8.jar"}
 
         if LooseVersion(self.version) <= LooseVersion('5.4.2'):  # log4j must be fixed till jmeter 5.4.2
             affected_names = ["log4j-core", "log4j-api", "log4j-slf4j-impl", "log4j-1.2-api"]
@@ -1616,15 +1616,14 @@ class JMeter(RequiredTool):
             return
 
         # Fix CVE-2016-10707 in jquery
-        jquery_src_dir = os.path.join(jmeter_dir, "jquery-dist-3.6.0")
+        jquery_src_dir = os.path.join(jmeter_dir, "jquery-dist-3.6.1")
         jquery_target_dir = os.path.join(jmeter_dir,
-            "bin/report-template/sbadmin2-1.0.7/bower_components/jquery")
-        jquery_tar = os.path.join(jmeter_dir, "jquery-dist-3.6.0.tar.gz")
-        self.__download_additions([["https://github.com/jquery/jquery-dist/archive/3.6.0.tar.gz",
+            "bin/report-template/sbadmin2-1.0.7/bower_components/jquery/")
+        jquery_tar = os.path.join(jmeter_dir, "jquery-dist-3.6.1.tar.gz")
+        self.__download_additions([["https://github.com/jquery/jquery-dist/archive/3.6.1.tar.gz",
                                     jquery_tar]])
         shutil.unpack_archive(jquery_tar, jmeter_dir)
         shutil.rmtree(jquery_target_dir, ignore_errors=True)
-        os.makedirs(jquery_target_dir)
         shutil.move(jquery_src_dir, jquery_target_dir)
         os.remove(jquery_tar)
 
