@@ -29,7 +29,8 @@ from urllib.error import URLError
 
 from bzt import NormalShutdown, ToolError, TaurusConfigError, TaurusInternalException
 from bzt.engine import Service, HavingInstallableTools, Singletone
-from bzt.utils import get_stacktrace, communicate
+from bzt.modules.javascript import NPMPackage, NPM
+from bzt.utils import get_stacktrace, communicate, BetterDict, TaurusCalledProcessError, Environment
 from bzt.utils import get_full_path, shutdown_process, shell_exec, RequiredTool, is_windows
 from bzt.utils import replace_in_config, JavaVM, Node, CALL_PROBLEMS, exec_and_communicate
 
@@ -41,45 +42,65 @@ if not is_windows():
 
 
 class PipInstaller(Service):
-    def __init__(self):
+    def __init__(self, packages=None, temp_flag=True):
         super(PipInstaller, self).__init__()
-        self.packages = []
-        self.temp = True
+        self.packages = packages or []
+        self.versions = BetterDict()
+        self.engine = None
+        self.temp = temp_flag
         self.target_dir = None
         self.interpreter = sys.executable
-        self.pip_cmd = [self.interpreter, "-m", "pip"]  # todo: change for win to bzt-pip
+        self.pip_cmd = [self.interpreter, "-m", "pip"]
 
-    def _install(self, packages):
-        if not packages:
-            self.log.debug("Nothing to install")
-            return
-        cmdline = self.pip_cmd + ["install", "-t", self.target_dir]
-        cmdline += self.packages
-        self.log.debug("pip-installer cmdline: '%s'" % ' '.join(cmdline))
-        out, err = exec_and_communicate(cmdline)
-        if err:
-            self.log.error("pip-installer stderr:\n%s" % err)
-        self.log.debug("pip-installer stdout: \n%s" % out)
+    def _check_pip(self):
+        cmdline = self.pip_cmd + ["--version"]
+        try:
+            exec_and_communicate(cmdline)
+        except TaurusCalledProcessError as exc:
+            self.log.debug(exc)
+            raise TaurusInternalException("pip module not found for interpreter %s" % self.interpreter)
 
-    def _missed(self, packages):
-        # todo: add version handling
+    def _get_installed(self):
         cmdline = self.pip_cmd + ["list"]
         out, _ = exec_and_communicate(cmdline)
-        list_of_installed = [line.split(' ')[0] for line in out.split('\n')[2:-1]]
+        out = out.split('\n')[2:-1]
+        return dict(zip([line.split(' ')[0] for line in out], [line.strip().split(' ')[-1] for line in out]))
 
+    def _missed(self, packages):
+        installed = self._get_installed()
         missed = []
         for package in packages:
-            if package not in list_of_installed:
+            if package not in installed or package in self.versions and installed[package] != self.versions[package]:
                 missed.append(package)
         return missed
 
-    def _uninstall(self, packages):
-        pass     # todo:
+    def _convert_config_versions(self):
+        """
+        extract from packages config:
+          packages:
+            - one
+            - two==0.0.0
+            - name: three
+              version: 0.0.0
+        and add to self.packages and self.versions
+        """
+        packages_list = self.parameters.get("packages", None)
+        if not packages_list:
+            return
 
-    def _reload(self, packages):
-        pass     # todo:
+        for package_data in packages_list:
+            package, version = None, None
+            if isinstance(package_data, dict):
+                package, version = package_data['name'], package_data.get("version", None)
+            elif isinstance(package_data, str):
+                package_params = package_data.split("==")
+                package, version = package_params[0], package_params[1] if len(package_params) > 1 else None
 
-    def prepare(self):
+            self.packages.append(package)
+            if version:
+                self.versions[package] = version
+
+    def prepare_pip(self):
         """
         pip-installer expect follow definition:
         - service pip-install
@@ -88,7 +109,9 @@ class PipInstaller(Service):
           - first_pkg
           - second_pkg
         """
-        self.packages = self.parameters.get("packages", self.packages)  # todo: add versions (dict format?)
+        self._check_pip()
+
+        self._convert_config_versions()
         if not self.packages:
             return
 
@@ -101,21 +124,95 @@ class PipInstaller(Service):
         if not os.path.exists(self.target_dir):
             os.makedirs(get_full_path(self.target_dir), exist_ok=True)
 
-        if self._missed(["pip"]):   # extend to windows (bzt-pip)
-            raise TaurusInternalException("pip module not found for interpreter %s" % self.interpreter)
-        self.packages = self._missed(self.packages)
-        if not self.packages:
-            return
+    def prepare(self):
+        self.prepare_pip()
+        if not self.all_packages_installed():
+            self.install()
 
-        self._install(self.packages)
+    def all_packages_installed(self):
+        self.packages = self._missed(self.packages)
+        self.versions = {package: self.versions[package]
+                         for package in self.versions.keys() if package in self.packages}
+        return False if self.packages else True
+
+    def install(self):
+        if not self.packages:
+            self.log.debug("Nothing to install")
+            return
+        cmdline = self.pip_cmd + ["install", "-t", self.target_dir]
+        for package in self.packages:
+            version = self.versions.get(package, None)
+            cmdline += [f"{package}=={version}"] if version else [package]
+        cmdline += ["--upgrade"]
+        self.log.debug("pip-installer cmdline: '%s'" % ' '.join(cmdline))
+        try:
+            out, err = exec_and_communicate(cmdline)
+        except TaurusCalledProcessError as exc:
+            self.log.debug(exc)
+            for line in exc.output.split('\n'):
+                if line.startswith("ERROR"):
+                    self.log.error(" ".join(line.split(" ")[1:]))
+            return
+        if "Successfully installed" in out:
+            self.log.info(out.split("\n")[-2])
+            for err_line in err.split("\n"):
+                if err_line.startswith('WARNING'):
+                    self.log.warning(" ".join(err_line.split(" ")[1:]))
+                if err_line.startswith('ERROR'):
+                    self.log.error(" ".join(err_line.split(" ")[1:]))
+        self.log.debug("pip-installer stdout: \n%s" % out)
+        if err:
+            self.log.debug("pip-installer stderr:\n%s" % err)
+
+    def get_version(self, package):
+        installed = self._get_installed()
+        return installed[package]
 
     def post_process(self):
-        if self.packages and self.temp and not is_windows():    # might be forbidden on win as tool still work
+        # might be forbidden on win as tool still work
+        if self.packages and self.temp and not is_windows() and os.path.exists(self.target_dir):
             self.log.debug("remove packages: %s" % self.packages)
 
             shutil.rmtree(self.target_dir)  # it removes all content of directory in reality, not only self.packages
 
 
+class PythonTool(RequiredTool):
+    PACKAGES = []
+
+    def __init__(self, engine, settings, **kwargs):
+        tool_path = engine.temp_pythonpath
+        version = settings.get("version", None)
+        dry_install = settings.get("dry-install", False)
+        super(PythonTool, self).__init__(tool_path=tool_path, version=version, dry_install=dry_install, **kwargs)
+        self.installer = PipInstaller(temp_flag=True if version else False)
+        self.installer.engine = engine
+        packages = copy.deepcopy(self.PACKAGES)
+        if self.version:
+            packages[0] += "==" + self.version
+        self.installer.parameters = BetterDict.from_dict({'packages': packages})
+
+    def check_if_installed(self):
+        self.log.debug(f"Checking {self.tool_name}.")
+        self.installer.prepare_pip()
+        result = self.installer.all_packages_installed()
+        if not result:
+            self.log.warning(f"{self.tool_name} check failed.")
+        return result
+
+    def install(self):
+        if self.dry_install:
+            self.log.info(f"Dry installation for {self.tool_name}")
+            return
+        self.log.debug(f"Installing {self.tool_name}.")
+        self.installer.install()
+
+    def get_version(self):
+        return self.installer.get_version(self.PACKAGES[0])
+
+    def post_process(self):
+        self.installer.post_process()
+
+
 class Unpacker(Service):
     UNPACK = 'unpacker'
     FILES = 'files'
@@ -134,7 +231,7 @@ class Unpacker(Service):
                 zip_file.extractall(self.engine.artifacts_dir)
 
             archive = os.path.basename(archive)
-            unpacked_list.append(archive[:-4])  # TODO: replace with top-level archive content
+            unpacked_list.append(archive[:-4])
 
         replace_in_config(self.engine.config, packed_list, unpacked_list, log=self.log)
 
@@ -267,33 +364,47 @@ class AndroidEmulatorLoader(Service):
 class AppiumLoader(Service):
     def __init__(self):
         super(AppiumLoader, self).__init__()
+        self.env = Environment(log=self.log)
         self.appium_process = None
         self.tool_path = ''
+        self.tools_dir = "~/.bzt/selenium-taurus/appium-server"
+        self.default_path = None
         self.startup_timeout = None
         self.addr = ''
         self.port = ''
         self.stdout = None
         self.stderr = None
+        self.appium_python = None
+        self.appium_server = None
 
     def prepare(self):
         self.startup_timeout = self.settings.get('timeout', 30)
         self.addr = self.settings.get('addr', '127.0.0.1')
         self.port = self.settings.get('port', 4723)
         self.tool_path = self.settings.get('path', 'appium')
+        self.tools_dir = get_full_path(self.settings.get("tools-dir", self.tools_dir))
+        self.default_path = os.path.join(self.tools_dir, "node_modules/.bin/appium")
+        self.env.add_path({"NODE_PATH": os.path.join(self.tools_dir, "node_modules")})
 
-        required_tools = [Node(log=self.log),
-                          JavaVM(log=self.log),
-                          Appium(tool_path=self.tool_path, log=self.log)]
+        self.install_required_tools()
 
+    def install_required_tools(self):
+        node = Node(env=self.env, log=self.log)
+        npm = NPM(env=self.env, log=self.log)
+        self.appium_python = AppiumPython(engine=self.engine, settings=self.settings, log=self.log)
+        self.appium_server = AppiumServer(path=self.tool_path, def_path=self.default_path, tools_dir=self.tools_dir,
+                                          node_tool=node, npm_tool=npm)
+        required_tools = [node, npm, JavaVM(log=self.log), self.appium_python, self.appium_server]
         for tool in required_tools:
             if not tool.check_if_installed():
                 tool.install()
 
     def startup(self):
-        self.log.debug('Starting appium...')
-        self.stdout = open(os.path.join(self.engine.artifacts_dir, 'appium.out'), 'ab')
-        self.stderr = open(os.path.join(self.engine.artifacts_dir, 'appium.err'), 'ab')
-        self.appium_process = shell_exec([self.tool_path], stdout=self.stdout, stderr=self.stderr)
+        self.log.debug('Starting Appium...')
+        self.stdout = open(os.path.join(self.engine.artifacts_dir, 'appium.out'), 'wt')
+        self.stderr = open(os.path.join(self.engine.artifacts_dir, 'appium.err'), 'wt')
+        self.appium_process = shell_exec([self.appium_server.tool_path, "--log-no-colors"],
+                                         stdout=self.stdout, stderr=self.stderr)
 
         start_time = time.time()
         while not self.tool_is_started():
@@ -328,14 +439,31 @@ class AppiumLoader(Service):
         if not os.stat(_file).st_size:
             os.remove(_file)
 
+    def post_process(self):
+        self.appium_python.post_process()
 
-class Appium(RequiredTool):
-    def __init__(self, **kwargs):
-        super(Appium, self).__init__(installable=False, **kwargs)
+
+class AppiumPython(PythonTool):
+    PACKAGES = ["Appium-Python-Client"]
+
+
+class AppiumServer(NPMPackage):
+    PACKAGE_NAME = "appium@1.22.0"
+
+    def __init__(self, path=None, def_path=None, **kwargs):
+        super(AppiumServer, self).__init__(**kwargs)
+        self.tool_path = path
+        self.default_path = def_path
 
     def check_if_installed(self):
         self.log.debug("Trying %s...", self.tool_name)
-        cmd = [self.tool_path, '--version']
+        if not self._check_path(self.tool_path):
+            self.tool_path = self.default_path
+            return self._check_path(self.default_path)
+        return True
+
+    def _check_path(self, path):
+        cmd = [path, '--version']
         try:
             out, err = exec_and_communicate(cmd)
         except CALL_PROBLEMS as exc:
@@ -397,7 +525,7 @@ class VirtualDisplay(Service, Singletone):
             self.log.info(msg, (width, height), self.virtual_display.new_display_var)
             VirtualDisplay.SHARED_VIRTUAL_DISPLAY = self.virtual_display
 
-            self.engine.shared_env.set({'DISPLAY': os.environ['DISPLAY']})   # backward compatibility
+            self.engine.shared_env.set({'DISPLAY': os.environ['DISPLAY']})  # backward compatibility
 
     def free_virtual_display(self):
         if self.virtual_display and self.virtual_display.is_alive():
